@@ -3,11 +3,11 @@ package ICC::Profile;
 use strict;
 use Carp;
 
-our $VERSION = 0.7;
+our $VERSION = 0.74;
 
-# revised 2018-09-09
+# revised 2019-08-10
 #
-# Copyright © 2004-2018 by William B. Birkett
+# Copyright © 2004-2019 by William B. Birkett
 
 # global variables
 our (@EXPORT, @EXPORT_OK, %EXPORT_TAGS);
@@ -72,7 +72,6 @@ BEGIN {
 
 	# optional modules
 	@opt = qw (
-		ICC::Support::Image
 		ICC::Support::Lapack
 		ICC::Support::Levmar
 	);
@@ -1097,17 +1096,20 @@ sub _readICCprofile {
 	} elsif (! ref($path)) {
 		
 		# replace '~~' with 'ICC' directory path
-		$path =~ s/^~~(?{ICC::Shared::getICCPath})/$^R/;
+		$path =~ s/^~~/ICC::Shared::getICCPath()/e;
 		
 		# filter file path
 		ICC::Shared::filterPath($path);
 		
+		# verify file path
+		-f $path or croak("$path is not a valid file path");
+		
 		# open the profile file
 		open($fh, '<', $path) or croak("unable to read profile from $path");
 		
 		# save path in object header
 		$self->[0]{'file_path'} = $path;
-
+		
 		# save file type in object header
 		$self->[0]{'file_type'} = 'prof';
 
@@ -1242,7 +1244,7 @@ sub _readICCprofile {
 				$tag->[3] = ICC::Profile::Generic->new_fh($self, $fh, $tag);
 				
 				# print message
-				print "tag type $type opened as generic\n";
+				# print "tag type $type opened as generic\n";
 				
 			}
 			
