@@ -3,11 +3,11 @@ package ICC::Profile::cvst;
 use strict;
 use Carp;
 
-our $VERSION = 0.41;
+our $VERSION = 0.48;
 
-# revised 2018-09-07
+# revised 2019-09-28
 #
-# Copyright © 2004-2018 by William B. Birkett
+# Copyright © 2004-2019 by William B. Birkett
 
 # add development directory
 use lib 'lib';
@@ -100,7 +100,7 @@ sub inv {
 }
 
 # create cvst object from ICC profile
-# assumes file handle is positoned at start of cvst data
+# assumes file handle is positioned at start of cvst data
 # header information must be read separately by the calling function
 # parameters: (ref_to_parent_object, file_handle, input_channels, output_channels)
 # returns: (ref_to_object)
@@ -357,7 +357,8 @@ sub jacobian {
 }
 
 # compute parametric Jacobian matrix
-# note: parameters are selected by the 'slice' array
+# parameters are selected by the 'slice' array -or- matrix
+# note: see 'cvst_parajac_matrix.plx' for explanation
 # parameters: (input_vector)
 # returns: (parametric_jacobian_matrix)
 sub parajac {
@@ -366,17 +367,26 @@ sub parajac {
 	my ($self, $in) = @_;
 
 	# local variables
-	my ($jac, $s, @pj);
+	my ($s, $type, @pj, $jac);
 
 	# verify curve object has 'parametric' method
 	($self->[1][0]->can('parametric')) or croak("curve object has no 'parametric' method");
 
-	# get slice array ref (if any)
+	# get 'slice' value
 	$s = $self->[0]{'slice'};
 
+	# determine 'slice' type (0 is undef, 1 is vector, 2 is matrix)
+	$type = ! defined($s) ? 0 : ICC::Shared::is_num_vector($s) ? 1 : ICC::Shared::is_num_matrix($s) && @{$s} == @{$self->[1]} ? 2 : croak("invalid slice for 'parajac' method");
+
+	# initialize matrix
+	$jac = [map {[]} 0 .. $#{$self->[1]}];
+
 	# for each channel
 	for my $i (0 .. $#{$self->[1]}) {
 		
+		# skip if slice empty
+		next if (($type == 1 && ! @{$s}) || ($type == 2 && ! @{$s->[$i]}));
+		
 		# get parametric partial derivatives
 		@pj = $self->[1][$i]->parametric($in->[$i]);
 		
@@ -386,12 +396,18 @@ sub parajac {
 			# if current channel
 			if ($j == $i) {
 				
-				# if slice defined
-				if ($s) {
+				# if vector slice
+				if ($type == 1) {
 					
 					# push slice parameters on matrix row
 					push(@{$jac->[$j]}, @pj[@{$s}]);
 					
+				# if matrix slice
+				} elsif ($type == 2) {
+					
+					# push slice parameters on matrix row
+					push(@{$jac->[$j]}, @pj[@{$s->[$i]}]);
+					
 				} else {
 					
 					# push all parameters on matrix row
@@ -401,12 +417,18 @@ sub parajac {
 				
 			} else {
 				
-				# if slice defined
-				if ($s) {
+				# if vector slice
+				if ($type == 1) {
 					
 					# push zeros on matrix row
 					push(@{$jac->[$j]}, (0) x @{$s});
 					
+				# if matrix slice
+				} elsif ($type == 2) {
+					
+					# push zeros on matrix row
+					push(@{$jac->[$j]}, (0) x @{$s->[$i]});
+					
 				} else {
 					
 					# push zeros on matrix row
@@ -610,6 +632,61 @@ sub apogee {
 
 }
 
+# write CGATS tone curve file
+# assumes curve domain/range is (0 - 1)
+# options parameter may be a hash reference or direction flag
+# hash keys: 'dir', 'steps'
+# direction: 0 - normal, 1 - inverse
+# parameters: (file_path, [options])
+sub cgats {
+
+	# get parameters
+	my ($self, $path, $opts) = @_;
+
+	# local variables
+	my ($dir, $steps, $mat, $fmt, $chart);
+
+	# process options
+	($dir, $steps) = _options($opts);
+
+	# filter path
+	ICC::Shared::filterPath($path);
+
+	# for each step
+	for my $i (0 .. $#{$steps}) {
+		
+		# add SampleID
+		$mat->[$i][0] = "A$i";
+		
+		# add input step value
+		$mat->[$i][1] = sprintf("\"%.2f\"", $steps->[$i]);
+		
+		# for each curve
+		for my $j (0 .. $#{$self->[1]}) {
+			
+			# add output step value
+			$mat->[$i][$j + 2] = sprintf("%.2f", 100 * ($self->[1][$j]->_transform($dir, $steps->[$i]/100)));
+			
+		}
+		
+	}
+
+	# make format string
+	$fmt = [qw(SampleID SAMPLE_NAME CMYK_C CMYK_M CMYK_Y CMYK_K), map {"SPOT_$_"} 1 .. ($#{$self->[1]} - 3)];
+
+	# make Chart object
+	$chart = ICC::Support::Chart->new($mat, {'format' => $fmt});
+
+	# add keywords
+	$chart->keyword('ORIGINATOR', '"PressCal"');
+	$chart->created(time);
+	$chart->keyword('LGOROWLENGTH', 5);
+
+	# write chart object
+	$chart->write($path);
+
+}
+
 # write device link profile containing tone curves
 # assumes curve domain/range is (0 - 1)
 # options parameter may be a hash reference or direction flag
@@ -663,7 +740,7 @@ sub device_link {
 	$profile = ICC::Profile->new({'class' => 'link', 'data' => $sig, 'PCS' => $sig, 'version' => '04200000'});
 
 	# add copyright tag
-	$profile->tag({'cprt' => ICC::Profile::mluc->new('en', 'US', 'Copyright (c) 2004-2018 by William B. Birkett')});
+	$profile->tag({'cprt' => ICC::Profile::mluc->new('en', 'US', 'Copyright (c) 2004-2019 by William B. Birkett')});
 
 	# add description tag
 	$profile->tag({'desc' => ICC::Profile::mluc->new('en', 'US', 'tone curves')});
@@ -700,7 +777,7 @@ sub device_link {
 
 }
 
-# write EFI tone curve file
+# write EFI (.vpc/.vcc) tone curve file
 # assumes curve domain/range is (0 - 1)
 # options parameter may be a hash reference or direction flag
 # hash keys: 'dir', 'steps'
@@ -727,9 +804,7 @@ sub efi {
 	if (($include) = grep {-d} map {File::Spec->catdir($_, 'ICC', 'Templates')} @INC) {
 		
 		# make a template processing object
-		$tt = Template->new({
-			'INCLUDE_PATH' => $include,
-		});
+		$tt = Template->new({'INCLUDE_PATH' => $include});
 		
 		# for each curve
 		for my $i (0 .. 7) {
@@ -872,7 +947,8 @@ sub fuji_xmf {
 # options parameter may be a hash reference or direction flag
 # hash key: 'dir'
 # direction: 0 - normal, 1 - inverse
-# note: values must be entered manually in Harlequin RIP
+# note: values must be entered manually in RIP
+# use 'navigator' method to make Postscript curves
 # parameters: (file_path, [options])
 sub harlequin {
 
@@ -996,8 +1072,9 @@ sub indigo {
 # write ISO 18620 (TED) tone curve file
 # assumes curve domain/range is (0 - 1)
 # options parameter may be a hash reference or direction flag
-# hash keys: 'dir', 'steps', 'inks', 'Creator', 'OperatorName',
-#	'PressName', 'MediaName', 'TransferCurveSetID', 'Side'
+# hash keys: 'dir', 'steps', 'inks', 'origin',
+#	'Creator', 'OperatorName', 'PressName', 'MediaName',
+#	'TransferCurveSetID', 'Side'
 # direction: 0 - normal, 1 - inverse
 # parameters: (file_path, [options])
 sub iso_18620 {
@@ -1006,7 +1083,7 @@ sub iso_18620 {
 	my ($self, $path, $opts) = @_;
 
 	# local variables
-	my ($dir, $steps, @inks, @attr);
+	my ($dir, $steps, @inks, $zflag);
 	my ($doc, $root, $t, $datetime, $curve, @out);
 
 	# process options
@@ -1026,9 +1103,6 @@ sub iso_18620 {
 	# filter path
 	ICC::Shared::filterPath($path);
 
-	# list of optional TransferCurveSet attributes
-	@attr = qw(Creator OperatorName PressName MediaName TransferCurveSetID Side);
-
 	# create XML document
 	$doc = XML::LibXML->createDocument('1.0', 'UTF-8');
 
@@ -1047,8 +1121,8 @@ sub iso_18620 {
 	# verify 'Side' attribute
 	(! defined($opts->{'Side'}) || $opts->{'Side'} eq 'Front' || $opts->{'Side'} eq 'Back') or croak('invalid \'Side\' attribute');
 
-	# for each optional attribute
-	for my $key (@attr) {
+	# for each optional TransferCurveSet attribute
+	for my $key (qw(Creator OperatorName PressName MediaName TransferCurveSetID Side)) {
 		
 		# if attribute contained in hash
 		if (defined($opts->{$key})) {
@@ -1063,6 +1137,9 @@ sub iso_18620 {
 	# set 'Creator' attribute, if undefined
 	$root->setAttribute('Creator', 'ICC-Profile Toolkit') if (! defined($opts->{'Creator'}));
 
+	# get 'origin' flag
+	$zflag = $opts->{'origin'} // 0;
+
 	# for each curve
 	for my $i (0 .. $#{$self->[1]}) {
 		
@@ -1073,7 +1150,7 @@ sub iso_18620 {
 		$curve->setAttribute('Separation', $inks[$i]);
 		
 		# compute and set the 'Curve' values
-		$curve->setAttribute('Curve', join(' ', map {sprintf("%f %f", $_/100, $self->[1][$i]->_transform($dir, $_/100))} @{$steps}));
+		$curve->setAttribute('Curve', join(' ', map {sprintf("%f %f", $_/100, ($_ == 0 && $zflag) ? 0 : $self->[1][$i]->_transform($dir, $_/100))} @{$steps}));
 		
 		# add curve node
 		$root->addChild($curve);
@@ -1088,6 +1165,95 @@ sub iso_18620 {
 
 }
 
+# write Xitron Navigator tone curve file
+# assumes curve domain/range is (0 - 1)
+# options parameter may be a hash reference or direction flag
+# hash key: 'dir', 'inks', 'name', 'colorspace'
+# direction: 0 - normal, 1 - inverse
+# note: makes a Postscript file for 'push calibration'
+# see Harlequin technical note Hqn081
+# parameters: (file_path, [options])
+sub navigator {
+
+	# get parameters
+	my ($self, $path, $opts) = @_;
+
+	# local variables
+	my ($dir, $steps, @inks, $tt, $include, $vars, $fh, $str);
+
+	# process options
+	($dir, $steps) = _options($opts);
+
+	# set ink colors
+	@inks = $#{$self->[1]} ? qw(Cyan Magenta Yellow Black) : qw(Black);
+
+	# for each curve
+	for my $i (0 .. $#{$self->[1]}) {
+		
+		# set ink value, defaults to 'inkN'
+		$inks[$i] = $opts->{'inks'}[$i] // $self->[0]{'inks'}[$i] // $inks[$i] // sprintf("ink%d", $i + 1);
+		
+	}
+
+	# filter path
+	ICC::Shared::filterPath($path);
+
+	# if ICC::Templates folder is found in @INC (may be relative)
+	if (($include) = grep {-d} map {File::Spec->catdir($_, 'ICC', 'Templates')} @INC) {
+		
+		# make a template processing object
+		$tt = Template->new({'INCLUDE_PATH' => $include});
+		
+		# set channels
+		$vars->{'channels'} = join(' ', map {"/$_"} @inks);
+		
+		# set channel colors
+		$vars->{'channelcolors'} = join(' ', map {"($_)"} @inks);
+		
+		# set number of channels
+		$vars->{'number'} = @inks;
+		
+		# set name
+		$vars->{'name'} = $opts->{'name'} // 'PressCal Calset ' . time();
+		
+		# set colorspace
+		$vars->{'colorspace'} = $opts->{'colorspace'} // 'DeviceCMYK';
+		
+		# open file handle to string
+		open($fh, '>', \$str);
+		
+		# for each channel
+		for my $i (0 .. $#{$self->[1]}) {
+			
+			# start curve
+			printf $fh "    /%s [\n", $inks[$i];
+			
+			# for each step
+			for my $j (0 .. $#{$steps}) {
+				
+				# print input and transformed values
+				printf $fh "    %.2f %% C%s\n", 100 * ($self->[1][$i]->_transform($dir, $steps->[$j]/100)), $steps->[$j];
+				
+			}
+			
+			# end curve
+			print $fh "    ]\n";
+			
+		}
+		
+		# add string to template hash
+		$vars->{'curves'} = $str;
+		
+		# close file handle
+		close($fh);
+		
+		# process the template
+		$tt->process('cvst_navigator.tt2', $vars, $path) || CORE::die $tt->error();
+		
+	}
+
+}
+
 # write Photoshop tone curve file
 # assumes curve domain/range is (0 - 1)
 # options parameter may be a hash reference or direction flag
@@ -1246,10 +1412,8 @@ sub photoshop {
 # assumes curve domain/range is (0 - 1)
 # options parameter may be a hash reference or direction flag
 # hash keys: 'dir', 'Comments', 'CurveSet', 'DefaultFrequency', 'DefaultMedium',
-#	'DefaultResolution', 'DefaultSpotFunction', 'Enabled', 'FirstName', 'FreqFrom',
-#	'FreqTo', 'FrequencyUsed', 'ID', 'Medium', 'MediumUsed', 'Resolution',
-#	'ResolutionUsed', 'ScreeningType', 'ScreeningTypeUsed', 'SpotFunction',
-#	'SpotFunctionMode', 'SpotFunctionUsed'
+#	'DefaultResolution', 'DefaultSpotFunction', 'Enabled', 'FirstName', 'FreqFrom', 'FreqTo',
+#	'ID', 'Medium', 'Resolution', 'ScreeningType', 'SpotFunction', 'SpotFunctionMode'
 # direction: 0 - normal, 1 - inverse
 # parameters: (file_path, [options])
 sub prinergy {
@@ -1258,124 +1422,114 @@ sub prinergy {
 	my ($self, $path, $opts) = @_;
 
 	# local variables
-	my ($dir, $steps, $prinergy, $key);
-	my ($fh, @time, $time, @month, $datetime);
-	my ($rs, @colors, @map);
+	my ($dir, $steps, @inks, $tt, $include, $vars, @time, @month, $fh, $rs, @map, $str);
 
 	# process options
 	($dir, $steps) = _options($opts);
 
-	# read prinergy hash template
-	$prinergy = YAML::Tiny->read(ICC::Shared::getICCPath('Preferences/Prinergy.yml'))->[0];
+	# set ink colors
+	@inks = $#{$self->[1]} ? qw(Cyan Magenta Yellow Black) : qw(Black);
 
-	# for each hash key
-	for my $key (keys(%{$prinergy})) {
+	# for each curve
+	for my $i (0 .. $#{$self->[1]}) {
 		
-		# set to options value, if defined
-		$prinergy->{$key} = $opts->{$key} if (defined($opts->{$key}));
-		
-		# set undefined values to null string
-		$prinergy->{$key} = '' if (! defined($prinergy->{$key}));
+		# set ink value, defaults to 'inkN'
+		$inks[$i] = $opts->{'inks'}[$i] // $self->[0]{'inks'}[$i] // $inks[$i] // sprintf("ink%d", $i + 1);
 		
 	}
 
 	# filter path
 	ICC::Shared::filterPath($path);
 
-	# open the file
-	open($fh, '>', $path) or croak("can't open $path: $!");
-
-	# disable :crlf translation
-	binmode($fh);
-
-	# set file creator and type (Windows NT SFM values)
-	ICC::Shared::setFile($path, 'LMAN', 'TEXT');
-
-	# get the time
-	@time = localtime(time);
-
-	# print time as string
-	$time = sprintf "%d/%d/%d %2.2d:%2.2d:%2.2d", $time[4]+1, $time[3], $time[5]+1900, $time[2], $time[1], $time[0];
-
-	# make array of months
-	@month = qw(January February March April May June July August September October November December);
-
-	# print datetime as string
-	$datetime = sprintf "%2.2d %s %d %2.2d:%2.2d:%2.2d", $time[3], $month[$time[4]], $time[5]+1900, $time[2], $time[1], $time[0];
-
-	# set output record separator (Windows CR-LF)
-	$rs = "\015\012";
-
-	# print Prinergy header
-	print $fh ";Creo Harmony Database File$rs";
-	print $fh ";1.07$rs";
-	print $fh ";$time$rs";
-	print $fh ";Next Calibration ID = 0001$rs$rs";
-
-	# print 'transfer' table info
-	printf $fh "[ E %s, %s %s %s %s %s ]$rs", $prinergy->{'FirstName'}, $prinergy->{'DefaultMedium'}, 'CMYK',
-	$prinergy->{'DefaultSpotFunction'}, $prinergy->{'DefaultFrequency'}, $prinergy->{'DefaultResolution'};
-	print $fh "FirstName = $prinergy->{'FirstName'}$rs";
-	print $fh "ID = $prinergy->{'ID'}$rs";
-	print $fh "Enabled = $prinergy->{'Enabled'}$rs";
-	print $fh "CurveSet = $prinergy->{'CurveSet'}$rs";
-	print $fh "DateTime = $datetime$rs";
-	printf $fh "Time = %d$rs", time;
-	print $fh "MediumUsed = $prinergy->{'MediumUsed'}$rs";
-	print $fh "Medium = $prinergy->{'Medium'}$rs";
-	print $fh "ScreeningTypeUsed = $prinergy->{'ScreeningTypeUsed'}$rs";
-	print $fh "ScreeningType = $prinergy->{'ScreeningType'}$rs";
-	print $fh "ResolutionUsed = $prinergy->{'ResolutionUsed'}$rs";
-	print $fh "Resolution = $prinergy->{'Resolution'}$rs";
-	print $fh "FrequencyUsed = $prinergy->{'FrequencyUsed'}$rs";
-	print $fh "FreqFrom = $prinergy->{'FreqFrom'}$rs";
-	print $fh "FreqTo = $prinergy->{'FreqTo'}$rs";
-	print $fh "SpotFunctionUsed = $prinergy->{'SpotFunctionUsed'}$rs";
-	print $fh "SpotFunction = $prinergy->{'SpotFunction'}$rs";
-	print $fh "SpotFunctionMode = $prinergy->{'SpotFunctionMode'}$rs";
-	print $fh "DefaultMedium = $prinergy->{'DefaultMedium'}$rs";
-	print $fh "DefaultResolution = $prinergy->{'DefaultResolution'}$rs";
-	print $fh "DefaultFrequency = $prinergy->{'DefaultFrequency'}$rs";
-	print $fh "DefaultSpotFunction = $prinergy->{'DefaultSpotFunction'}$rs";
-
-	# set color names
-	@colors = qw(Cyan Magenta Yellow Black);
-
-	# set color map (KCMY)
-	@map = (3, 0, 1, 2);
-
-	# for each curve
-	for my $i (0 .. 3) {
-
-		# print curve dropoff
-		printf $fh "Curve%d DropOff = %d$rs", $i + 1, 0;
-
-		# print curve color
-		printf $fh "Curve%d Color = %s$rs", $i + 1, $colors[$map[$i]];
-
-		# print curve start
-		printf $fh "Curve%d = ", $i + 1;
-
-		# print curve points
-		for my $t (@{$steps}) {
+	# if ICC::Templates folder is found in @INC (may be relative)
+	if (($include) = grep {-d} map {File::Spec->catdir($_, 'ICC', 'Templates')} @INC) {
 		
-			# print curve values
-			printf $fh "%d %d ", 1E7 * $t/100 + 0.5, 1E7 * $self->[1][$map[$i]]->_transform($dir, $t/100) + 0.5;
+		# make a template processing object
+		$tt = Template->new({'INCLUDE_PATH' => $include});
 		
+		# copy options hash
+		$vars = Storable::dclone($opts);
+		
+		# set time
+		$vars->{'Time'} = time();
+		
+		# get localtime
+		@time = localtime($vars->{'Time'});
+		
+		# set date as string
+		$vars->{'date'} = sprintf "%d/%d/%d %2.2d:%2.2d:%2.2d", $time[4] + 1, $time[3], $time[5] + 1900, $time[2], $time[1], $time[0];
+
+		# make array of months
+		@month = qw(January February March April May June July August September October November December);
+
+		# set DateTime as string
+		$vars->{'DateTime'} = sprintf "%2.2d %s %d %2.2d:%2.2d:%2.2d", $time[3], $month[$time[4]], $time[5] + 1900, $time[2], $time[1], $time[0];
+		
+		# set defaults
+		$vars->{'FirstName'} = $vars->{'FirstName'} // 'PressCal';
+		$vars->{'ID'} = $vars->{'ID'} // '0001';
+		$vars->{'Enabled'} = $vars->{'Enabled'} // 'FALSE';
+		$vars->{'CurveSet'} = $vars->{'CurveSet'} // 'CmykCurves';
+		$vars->{'SpotFunctionMode'} = $vars->{'SpotFunctionMode'} // 'UserDefined';
+		
+		# set true or false
+		$vars->{'MediumUsed'} = defined($vars->{'Medium'}) ? 'TRUE' : 'FALSE';
+		$vars->{'ScreeningTypeUsed'} = defined($vars->{'ScreeningType'}) ? 'TRUE' : 'FALSE';
+		$vars->{'ResolutionUsed'} = defined($vars->{'Resolution'}) ? 'TRUE' : 'FALSE';
+		$vars->{'FrequencyUsed'} = (defined($vars->{'FreqFrom'}) && defined($vars->{'FreqFrom'})) ? 'TRUE' : 'FALSE';
+		$vars->{'SpotFunctionUsed'} = defined($vars->{'SpotFunction'}) ? 'TRUE' : 'FALSE';
+		
+		# set combined description
+		$vars->{'description'} = join(' ', grep {$_} @{$vars}{qw(FirstName Medium CurveSet FreqFrom Resolution)});
+
+		# open file handle to string
+		open($fh, '>', \$str);
+		
+		# disable :crlf translation
+		binmode($fh);
+		
+		# set output record separator (Windows CR-LF)
+		$rs = "\015\012";
+
+		# set color map (KCMY + spot)
+		@map = (3, 0, 1, 2, 4 .. 15);
+		
+		# for each channel
+		for my $i (0 .. $#{$self->[1]}) {
+			
+			# print curve dropoff
+			printf $fh "Curve%d DropOff = %d$rs", $i + 1, 0;
+			
+			# print curve color
+			printf $fh "Curve%d Color = %s$rs", $i + 1, $inks[$map[$i]];
+			
+			# print curve start
+			printf $fh "Curve%d = ", $i + 1;
+			
+			# print curve points
+			for my $t (@{$steps}) {
+				
+				# print curve values
+				printf $fh "%d %d ", 1E7 * $t/100 + 0.5, 1E7 * $self->[1][$map[$i]]->_transform($dir, $t/100) + 0.5;
+				
+			}
+			
+			# print curve end
+			print $fh "$rs";
+			
 		}
-
-		# print curve end
-		print $fh "$rs";
-
+		
+		# add string to template hash
+		$vars->{'curves'} = $str;
+		
+		# close the file
+		close($fh);
+		
+		# process the template
+		$tt->process('cvst_prinergy.tt2', $vars, $path) || CORE::die $tt->error();
+		
 	}
 
-	# print trailing comments
-	print $fh "Comments = $prinergy->{'Comments'}$rs";
-	print $fh "$rs";
-
-	# close the file
-	close($fh);
-
 }
 
 # write Rampage tone curve file set
@@ -1489,6 +1643,62 @@ sub rampage {
 	
 }
 
+# write Xitron Sierra tone curve file
+# assumes curve domain/range is (0 - 1)
+# options parameter may be a hash reference or direction flag
+# hash key: 'dir'
+# direction: 0 - normal, 1 - inverse
+# parameters: (file_path, [options])
+sub sierra {
+
+	# get parameters
+	my ($self, $path, $opts) = @_;
+
+	# local variables
+	my ($dir, $steps, $fh, $rs, @colors, @Tdot);
+
+	# process options
+	($dir, $steps) = _options($opts);
+
+	# filter path
+	ICC::Shared::filterPath($path);
+
+	# open the file
+	open($fh, '>', $path) or croak("can't open $path: $!");
+
+	# disable :crlf translation
+	binmode($fh);
+
+	# set output record separator (Windows CR-LF)
+	$rs = "\015\012";
+
+	# set color list
+	@colors = qw(Cyan Magenta Yellow Black);
+
+	# print colors
+	print $fh join(';', @colors), $rs;
+
+	# for each step
+	for my $j (0 .. $#{$steps}) {
+		
+		# for each channel
+		for my $i (0 .. 3) {
+			
+			# compute transformed dot value
+			$Tdot[$i] = sprintf("%.4f", 100 * ($self->[1][$i]->_transform($dir, $steps->[$j]/100)));
+			
+		}
+		
+		# print transformed values
+		print $fh join(';', @Tdot), $rs;
+		
+	}
+
+	# close the file
+	close($fh);
+
+}
+
 # write Trueflow tone curve file
 # assumes curve domain/range is (0 - 1)
 # options parameter may be a hash reference or direction flag
@@ -1586,62 +1796,6 @@ sub trueflow {
 
 }
 
-# write Xitron Sierra tone curve file
-# assumes curve domain/range is (0 - 1)
-# options parameter may be a hash reference or direction flag
-# hash key: 'dir'
-# direction: 0 - normal, 1 - inverse
-# parameters: (file_path, [options])
-sub xitron {
-
-	# get parameters
-	my ($self, $path, $opts) = @_;
-
-	# local variables
-	my ($dir, $steps, $fh, $rs, @colors, @Tdot);
-
-	# process options
-	($dir, $steps) = _options($opts);
-
-	# filter path
-	ICC::Shared::filterPath($path);
-
-	# open the file
-	open($fh, '>', $path) or croak("can't open $path: $!");
-
-	# disable :crlf translation
-	binmode($fh);
-
-	# set output record separator (Windows CR-LF)
-	$rs = "\015\012";
-
-	# set color list
-	@colors = qw(Cyan Magenta Yellow Black);
-
-	# print colors
-	print $fh join(';', @colors), $rs;
-
-	# for each step
-	for my $j (0 .. $#{$steps}) {
-		
-		# for each channel
-		for my $i (0 .. 3) {
-			
-			# compute transformed dot value
-			$Tdot[$i] = sprintf("%.4f", 100 * ($self->[1][$i]->_transform($dir, $steps->[$j]/100)));
-			
-		}
-		
-		# print transformed values
-		print $fh join(';', @Tdot), $rs;
-		
-	}
-
-	# close the file
-	close($fh);
-
-}
-
 # write tab delimited text tone curve file
 # assumes curve domain/range is (0 - 1)
 # options parameter may be a hash reference or direction flag
@@ -1701,16 +1855,17 @@ sub text {
 # graph tone curves
 # assumes curve domain/range is (0 - 1)
 # options parameter may be a hash reference or direction flag
-# hash keys: 'dir', 'composite', 'titles', 'inks', 'files'
+# hash keys: 'dir', 'lib', 'composite', 'titles', 'inks', 'files', 'open'
 # direction: 0 - normal, 1 - inverse
 # parameters: (folder_path, [options])
+# returns: (graph_path_list)
 sub graph {
 
 	# get parameters
 	my ($self, $path, $opts) = @_;
 
 	# local variables
-	my ($dir, $include, $tt, $vars, @colors, @inks, @data, $file, $s, $lib, $js0, $js1, $html);
+	my ($dir, $include, $tt, $vars, $min, $max, @inks, %exc, @colors, @data, @tooltips, $file, $s, @html);
 
 	# process options
 	($dir) = _options($opts);
@@ -1722,37 +1877,24 @@ sub graph {
 		ICC::Shared::filterPath($path);
 		
 		# make a template processing object
-		$tt = Template->new({
-			'INCLUDE_PATH' => $include,
-			'OUTPUT_PATH' => $path,
-		});
+		$tt = Template->new({'INCLUDE_PATH' => $include, 'OUTPUT_PATH' => $path});
 		
 		# if gray scale curve
 		if ($#{$self->[1]} == 0) {
 			
-			# set colors
-			@colors = qw(black);
-			@inks = qw(grayscale);
-			
-		# if duotone curve
-		} elsif ($#{$self->[1]} == 1) {
-			
-			# set colors (like Excel)
-			@colors = qw(cyan red);
+			# set default ink color
+			@inks = qw(gray);
 			
 		# if RGB curves
 		} elsif ($#{$self->[1]} == 2) {
 			
-			# set colors
-			@colors = @inks = qw(red green blue);
+			# set default ink colors
+			@inks = qw(red green blue);
 			
 		# if CMYK+ curves
 		} else {
 			
-			# set colors
-			@colors = qw(cyan magenta yellow black orange green blue);
-			$colors[2] = '#cc0'; # dark yellow
-			$colors[4] = '#f80'; # orange
+			# set default ink colors
 			@inks = qw(cyan magenta yellow black);
 			
 		}
@@ -1765,6 +1907,19 @@ sub graph {
 			
 		}
 		
+		# graph color exceptions
+		%exc = ('yellow' => '#ee0', 'orange' => '#f80', 'violet' => '#80f', 'gray' => '#777');
+		
+		# get graph colors, mapping exceptions
+		@colors = map {$exc{$_} // $_} map {m/^ink_/ ? 'gray' : $_} @inks;
+		
+		# set RGraph library folder
+		$vars->{'libjs'} = $opts->{'lib'} // 'lib';
+		
+		# set yaxis scale
+		$vars->{'yscalemin'} = $min = $opts->{'yscalemin'} // 0.0;
+		$vars->{'yscalemax'} = $max = $opts->{'yscalemax'} // 1.0;
+		
 		# if 'composite' curve
 		if ($opts->{'composite'}) {
 			
@@ -1775,7 +1930,7 @@ sub graph {
 				@data = map {sprintf("%.3f", $self->[1][$i]->_transform($dir, $_/100))} (0 .. 100);
 				
 				# clip the data
-				@data = map {$_ < 0.0 ? '0.000' : $_ > 1.0 ? '1.000' : $_} @data;
+				@data = map {$_ < $min ? $min : $_ > $max ? $max : $_} @data;
 				
 				# make javascript string of curve data
 				$s->[$i] = '[' . join(', ', @data) . ']';
@@ -1785,6 +1940,9 @@ sub graph {
 			# make composite javascript string of curve data
 			$vars->{'data'} = '[' . join(', ', @{$s}) . ']';
 			
+			# disable tooltips
+			$vars->{'tooltips'} = '[]';
+			
 			# set graph title
 			$vars->{'title'} = $opts->{'titles'}[0] // "composite tone curves";
 			
@@ -1806,11 +1964,17 @@ sub graph {
 				@data = map {sprintf("%.3f", $self->[1][$i]->_transform($dir, $_/100))} (0 .. 100);
 				
 				# clip the data
-				@data = map {$_ < 0.0 ? '0.000' : $_ > 1.0 ? '1.000' : $_} @data;
+				@data = map {$_ < $min ? $min : $_ > $max ? $max : $_} @data;
 				
 				# make javascript string of curve data
 				$vars->{'data'} = '[[' . join(', ', @data) . ']]';
 				
+				# compute tooltips array
+				@tooltips = map {$_ % 5 ? 'null' : sprintf("'%d%% ➔ %.1f%%'", $_, 100 * $data[$_])} (0 .. 100);
+				
+				# make tooltips
+				$vars->{'tooltips'} = '[' . join(', ', @tooltips) . ']';
+			
 				# set graph title
 				$vars->{'title'} = $opts->{'titles'}[$i] // "$inks[$i] tone curve";
 				
@@ -1827,50 +1991,19 @@ sub graph {
 			
 		}
 		
-		# make path to 'lib' folder
-		$lib = $path . ($^O eq 'MSWin32' ? '\lib' : '/lib');
-		
-		# make 'lib' folder, if none
-		File::Path::make_path($lib);
-		
-		# match 'ICC' folder path
-		$include =~ m/^(.*)Templates$/;
-		
-		# if Windows OS
-		if ($^O eq 'MSWin32') {
-			
-			# make Rgraph Javascript paths
-			$js0 = '\Javascripts\rgraph\RGraph.svg.common.core.js';
-			$js1 = '\Javascripts\rgraph\RGraph.svg.line.js';
-			
-			# copy Rgraph Javascripts to 'lib' folder
-			qx(copy "$1$js0" $lib) if (! -f "$lib$js0");
-			qx(copy "$1$js1" $lib) if (! -f "$lib$js1");
-			
-		} else {
-			
-			# copy Rgraph Javascripts to 'lib' folder (Unix)
-			qx(cp -n "$1/Javascripts/rgraph/RGraph.svg.common.core.js" $lib);
-			qx(cp -n "$1/Javascripts/rgraph/RGraph.svg.line.js" $lib);
-			
-		}
-		
 		# if 'composite' curve
 		if ($opts->{'composite'}) {
 			
 			# if Windows OS
 			if ($^O eq 'MSWin32') {
 				
-				# make html path
-				$html = "$path\\$file.html";
-				
-				# open graph file in browser
-				qx(start $html);
+				# set file list
+				@html = ("$path\\$file.html");
 				
 			} else {
 				
-				# open graph file in browser
-				qx(open "$path/$file.html");
+				# set file list
+				@html = ("$path/$file.html");
 				
 			}
 			
@@ -1885,28 +2018,142 @@ sub graph {
 				# if Windows OS
 				if ($^O eq 'MSWin32') {
 					
-					# make html path
-					$html = "$path\\$file.html";
-					
-					# open graph file in browser
-					qx(start $html);
+					# add path to file list
+					push(@html, "$path\\$file.html");
 					
 				} else {
 					
-					# open graph file in browser
-					qx(open "$path/$file.html");
+					# add path to file list
+					push(@html, "$path/$file.html");
 					
 				}
 				
-				# pause for first file (Firefox)
-				sleep(1) if ($i == 0);
-				
 			}
 			
 		}
 		
 	}
-	
+
+	# open files, if enabled
+	open_files(\@html) if ($opts->{'open'} // 1);
+
+	# return
+	return(@html);
+
+}
+
+# display graphs in web browser
+# parameters: (ref_to_file_list)
+sub open_files {
+
+	# get file list
+	my $files = shift();
+
+	# local parameters
+	my ($RGraph, $vol, $dir, $file, $lib, $app, @fox, @pid, @esc, $esc0, $flag, $timeout);
+
+	# find RGraph folder path in @INC (may be relative)
+	($RGraph) = grep {-d} map {File::Spec->catdir($_, 'ICC', 'JavaScripts', 'RGraph')} @INC;
+
+	# if valid file list and RGraph folder found
+	if (ref($files) eq 'ARRAY' && defined($files->[0]) && -f $files->[0] && defined($RGraph)) {
+		
+		# split first file path
+		($vol, $dir, $file) = File::Spec->splitpath($files->[0]);
+		
+		# make 'lib' folder path
+		$lib = File::Spec->catdir($vol, $dir, 'lib');
+		
+		# if macOS
+		if ($^O eq 'darwin') {
+			
+			# copy RGraph JavaScripts to 'lib' folder
+			qx(cp -Rp '$RGraph/' '$lib');
+			
+			# escape the file paths
+			@esc = map {quotemeta()} @{$files};
+			
+			# get default app (using JXA)
+			$app = qx(osascript -l JavaScript -e "Application('System Events').files.byName('$files->[0]').defaultApplication.name()");
+			
+			# remove endline
+			chomp($app);
+			
+			# if default app is Firefox
+			if ($app eq 'Firefox.app') {
+				
+				# get first file path
+				$esc0 = shift(@esc);
+				
+				# open first graph 
+				qx(open $esc0);
+				
+				# if more graphs
+				if (@esc) {
+					
+					# set timeout (5 secs)
+					$timeout = time() + 5;
+					
+					# loop until we get Firefox pid -or- timeout
+					while (! @fox && time() < $timeout) {
+						
+						# get Firefox pid
+						@fox = split(/\s+/, qx(pgrep firefox));
+						
+					}
+					
+					# set flag
+					$flag = 1;
+					
+					# loop until flag is cleared -or- timeout
+					while ($flag && time() < $timeout) {
+						
+						# if 4 or more child processes
+						if ((@pid = split(/\s+/, qx(pgrep -P $fox[0]))) > 3) {
+							
+							# for each child process
+							for (@pid) {
+								
+								# clear flag if ps command contains '-sbAllowFileAccess'
+								$flag = 0 if (qx(ps -p $_ -o command) =~ m/-sbAllowFileAccess/m);
+								
+							}
+							
+						}
+						
+					}
+					
+					# open remaining graphs
+					qx(open @esc);
+					
+				}
+				
+			} else {
+				
+				# open all graphs
+				qx(open @esc);
+				
+			}
+			
+		# if Windows OS
+		} elsif ($^O eq 'MSWin32') {
+			
+			# copy RGraph JavaScripts to 'lib' folder
+			qx(xcopy /I "$RGraph\\" "$lib\\");
+			
+			print "to be implemented\n\n"; ###########
+			
+		} else {
+			
+			print "unsupported OS\n\n";
+			
+		}
+		
+	}
+
+	# return
+	return();
+
 }
 
 # normalize all curve objects
@@ -2346,20 +2593,22 @@ sub _options {
 	my ($dir, $steps, @ctx, $caller, $n);
 
 	# make hash of standard step ramps (method_name => [ref_to_steps_array, custom_flag])
-	# steps array contains device values, flag indicate values may be custom
+	# steps array contains device values (%), flag indicate values may be custom
 	state $std = {
 		'apogee' => [[0 .. 6, (map {5 * $_} 2 .. 18), 94 .. 100], 1],
 		'device_link' => [[], 0],
+		'cgats' => [[0, 2, 4, 6, 8, (map {5 * $_} 2 .. 19), 98, 100], 1], # P2P51 ramp
 		'efi' => [[0 .. 3, map {5 * $_} 1 .. 20], 0],
 		'fuji_xmf' => [[0 .. 5, 10, 20, 25, 30, 40, 50, 60, 70, 75, 80, 90, 95 .. 100], 0],
 		'harlequin' => [[100, 95, 90, 85, 80, 70, 60, 50, 40, 30, 20, 15, 10, 8, 6, 4, 2, 0], 0],
 		'indigo' => [[map {5 * $_} 0 .. 20], 0],
 		'iso_18620' => [[0, 1, 2, 5, (map {10 * $_} (1 .. 9)), 95, 100], 1],
+		'navigator' => [[100, 95, 90, 85, 80, 70, 60, 50, 40, 30, 20, 15, 10, 8, 6, 4, 2, 0], 0],
 		'photoshop' => [[], 1],
 		'prinergy' => [[0 .. 100], 0],
 		'rampage' => [[0, 1, 3, (map {5 * $_} 1 .. 19), 97, 99, 100], 0],
+		'sierra' => [[0 .. 5, 10, 20, 25, 30, 40, 50, 60, 70, 75, 80, 90, 95 .. 100], 0],
 		'trueflow' => [[0, 2, 5, (map {10 * $_} 1 .. 9), 95, 98, 100], 0],
-		'xitron' => [[0 .. 5, 10, 20, 25, 30, 40, 50, 60, 70, 75, 80, 90, 95 .. 100], 0],
 		'text' => [[map {$_ * 5} 0 .. 20], 1],
 		'graph' => [[], 0],
 	};
@@ -2509,7 +2758,7 @@ sub _read_text {
 	@data = grep {$n == @{$_}} @data;
 
 	# verify data table size
-	(@data > 1 && @{$data[0]} > 1) || return(0);
+	(@data > 1 && @{$data[0]} > 1) or return(0);
 
 	# sort by first value in each line
 	@data = sort {$a->[0] <=> $b->[0]} @data;
@@ -2552,7 +2801,7 @@ sub _read_iso_18620 {
 	my ($dom, $root, @obj, $k, @sep, $curve, @xy, @x, @y);
 
 	# parse ISO 18620 document
-	eval{$dom = XML::LibXML->load_xml('IO' => $fh)} || return('failed parsing ISO 18620 document');
+	eval{$dom = XML::LibXML->load_xml('IO' => $fh)} or return(0);
 
 	# get root element
 	$root = $dom->documentElement();
@@ -2607,24 +2856,106 @@ sub _read_iso_18620 {
 	}
 
 	# add ink sequence
-	$self->[0]{'inks'} = [@sep];
+	$self->[0]{'inks'} = \@sep;
 
 	# return
-	return($k ? 1 : 0);
+	return($k);
+
+}
+
+# read curves from Esko .icpro/.dgc file set
+# returns true if successful
+# parameters: (ref_to_object, file_handle, path)
+# returns: (flag)
+sub _read_icpro {
+
+	# get parameters
+	my ($self, $fh, $path) = @_;
+
+	# local variables
+	my ($dom, $root, @obj, $k, @sep, $dgc, $curve);
+	my ($vol, $dir, $file, $fh2, $buf, $ptr, $n, $max, @data);
+
+	# split .icpro path
+	($vol, $dir, $file) = File::Spec->splitpath($path);
+
+	# parse .icpro document
+	eval{$dom = XML::LibXML->load_xml('IO' => $fh)} or return(0);
+
+	# get root element
+	$root = $dom->documentElement();
+
+	# get all 'ink' nodes
+	@obj = $root->findnodes('*/ink');
+
+	# init curve counter
+	$k = 0;
+
+	# for each 'ink' node
+	for my $s (@obj) {
+		
+		# if there is a 'dgc' node
+		if (($dgc) = $s->findnodes('dgc')) {
+			
+			# get the 'fileName' attribute
+			$file = $dgc->getAttribute('fileName');
+			
+			# concatenate file path
+			$path = File::Spec->catfile($vol, $dir, $file);
+			
+			# open the file (read-only)
+			open($fh2, '<', $path) or croak("$! when opening file $path");
+			
+			# seek table 4 index
+			seek($fh2, 0x0000020C, 0);
+			
+			# read index to table 4
+			read($fh2, $buf, 12);
+			
+			# unpack table pointer, number of points
+			($ptr, $n, $max) = unpack('N*', $buf);
+			
+			# seek table 4
+			seek($fh2, $ptr, 0);
+			
+			# read table data
+			read($fh2, $buf, $n * 4);
+			
+			# unpack table data
+			@data = map {$_/$max} unpack('N*', $buf);
+			
+			# close file
+			close($fh2);
+			
+			# make 'curv' object
+			$self->[1][$k] = ICC::Profile::curv->new(\@data);
+			
+			# get the 'inkName' attribute
+			$sep[$k++] = $s->getAttribute('inkName');
+			
+		}
+		
+	}
+
+	# add ink sequence
+	$self->[0]{'inks'} = \@sep;
+
+	# return
+	return($k);
 
 }
 
 # read curves from file
-# file path to 'iso_18620', 'store', or 'text' format curves
+# file path to 'iso_18620', 'icpro', 'text' or 'store' format curves
 # parameters: (ref_to_object, file_path)
 sub _new_from_file {
-	
+
 	# get parameters
 	my ($self, $path) = @_;
-	
+
 	# local variables
 	my ($fh, $buf, $result, $obj);
-	
+
 	# filter path name
 	ICC::Shared::filterPath($path);
 
@@ -2640,8 +2971,8 @@ sub _new_from_file {
 	# reset file pointer
 	seek($fh, 0, 0);
 
-	# if an XML file
-	if ($buf =~ m/<\?xml.*\?>/) {
+	# if an ISO 18620 (.ted) file
+	if ($buf =~ m/<\?xml.*\?>/ && $buf =~ m/ISO18620/) {
 		
 		# read ISO 18620 file
 		_read_iso_18620($self, $fh) or croak("failed parsing ISO 18620 (XML) file $path");
@@ -2649,6 +2980,15 @@ sub _new_from_file {
 		# save file type
 		$self->[0]{'file_type'} = 'ISO_18620';
 		
+	# if an Esko .icpro file
+	} elsif ($buf =~ m/<\?xml.*\?>/ && $buf =~ m/colDgc_xml/) {
+		
+		# read .icpro/.dgc file set
+		_read_icpro($self, $fh, $path) or croak("failed parsing .icpro/.dgc file set $path");
+		
+		# save file type
+		$self->[0]{'file_type'} = 'ESKO_ICPRO';
+		
 	# if a Storable file
 	} elsif ($buf =~ m/ICC::Profile::cvst/) {
 		
