@@ -12,11 +12,11 @@ use Storable;
 use YAML::Tiny;
 use Exporter qw(import);
 
-our $VERSION = 0.57;
+our $VERSION = 0.58;
 
-# revised 2018-09-09
+# revised 2019-11-16
 #
-# Copyright © 2004-2018 by William B. Birkett
+# Copyright © 2004-2019 by William B. Birkett
 
 # this module contains common methods, constants and functions
 
@@ -45,7 +45,7 @@ BEGIN {
 # list of constants and functions to export
 our @EXPORT = qw(D50 d50 d50P PI radian ln10 xyz2Lab Lab2xyz xyz2Lxyz Lxyz2xyz Lab2Lxyz Lxyz2Lab
 XYZ2Lab Lab2XYZ XYZ2Lxyz Lxyz2XYZ XYZ2xyz xyz2XYZ XYZ2xyY xyY2XYZ x2L L2x dLdx dxdL XYZ2W xyz2dwv
-xyz2Lab_jac Lab2xyz_jac dEab dEcmc dE94 dE00 dE99 dCh CCT CCT2 bbrad bbxy bbuv XYZ2ucs xy2ucs daylight
+xyz2Lab_jac Lab2xyz_jac dEab dEcmc dE94 dE00 dE99 dH dLCh dLab CCT CCT2 bbrad bbxy bbuv XYZ2ucs xy2ucs daylight
 linear linear_matrix cspline cspline_matrix lagrange lagrange_matrix dotProduct crossProduct
 flatten clip_struct round s15f162v v2s15f16 makeProfileFolder getICCPath filterPath setFile parse_tokens
 is_vector is_num_vector is_matrix is_num_matrix);
@@ -670,12 +670,38 @@ sub dE99 {
 
 }
 
-# compute ∆Ch chroma difference (aka ∆F)
+# compute ∆H color difference
 # parameters: (L*a*b*_1, L*a*b*_2)
-sub dCh {
+# returns: (∆H)
+sub dH {
 
-	# return chroma difference
-	return(sqrt(($_[1] - $_[4])**2 + ($_[2] - $_[5])**2));
+	# compute chroma values
+	my $C0 = sqrt($_[1]**2 + $_[2]**2);
+	my $C1 = sqrt($_[4]**2 + $_[5]**2);
+
+	# return ∆H
+	return(sqrt(abs(($_[1] - $_[4])**2 + ($_[2] - $_[5])**2 - ($C1 - $C0)**2)));
+
+}
+
+# compute ∆L*, ∆Ch (aka ∆F) color differences
+# as used to determine G7 grayscale compliance
+# parameters: (L*a*b*_1, L*a*b*_2)
+# returns: (∆L*, ∆Ch)
+sub dLCh {
+
+	# return ∆L*, ∆Ch
+	return($_[0] - $_[3],  sqrt(($_[1] - $_[4])**2 + ($_[2] - $_[5])**2));
+
+}
+
+# compute ∆L*, ∆a*, ∆b* color differences
+# parameters: (L*a*b*_1, L*a*b*_2)
+# returns: (∆L*, ∆a*, ∆b*)
+sub dLab {
+
+	# return ∆L*, ∆a*, ∆b*
+	return($_[0] - $_[3], $_[1] - $_[4], $_[2] - $_[5]);
 
 }
 
@@ -2301,9 +2327,12 @@ sub filterPath {
 		
 	}
 
+	# return
+	return();
+
 }
 
-# set Mac OSX file creator and type
+# set macOS file creator and type
 # calls the 'SetFile' utility, if present
 # parameters: (file_path, creator, type)
 sub setFile {
@@ -2317,17 +2346,11 @@ sub setFile {
 	# escape spaces and special characters
 	$path =~ s/([^\w\-\/+.@])/\\$1/g;
 
-	# if SetFile in /usr/bin directory
-	if (-f '/usr/bin/Setfile') {
+	# if SetFile in /Library/Developer directory
+	if (-f '/Library/Developer/CommandLineTools/usr/bin/SetFile') {
 		
 		# make system call
-		qx(/usr/bin/Setfile -c $creator -t $type $path);
-		
-	# if SetFile in /Developer/usr/bin directory
-	} elsif (-f '/Developer/usr/bin/SetFile') {
-		
-		# make system call
-		qx(/Developer/usr/bin/SetFile -c $creator -t $type $path);
+		qx(/Library/Developer/CommandLineTools/usr/bin/SetFile -c $creator -t $type $path);
 		
 	}
 
@@ -2344,7 +2367,7 @@ sub parse_tokens {
 	# get parameter
 	my $str = shift();
 
-	# local parameters
+	# local variables
 	my ($nest, $j, $char, @px, $sub, @pars, @eval);
 
 	# return if parameter undefined
@@ -2832,7 +2855,7 @@ sub _Lab2DIN99 {
 package Math::Matrix;
 
 # print object contents to string
-# format is an array structure
+# format parameter is an array structure
 # parameter: ([format])
 # returns: (string)
 sub sdump {
