@@ -3,11 +3,11 @@ package ICC::Support::PCS;
 use strict;
 use Carp;
 
-our $VERSION = 0.74;
+our $VERSION = 0.75;
 
-# revised 2018-05-12
+# revised 2019-01-07
 #
-# Copyright © 2004-2018 by William B. Birkett
+# Copyright © 2004-2019 by William B. Birkett
 
 # add development directory
 use lib 'lib';
@@ -231,8 +231,8 @@ sub scale {
 		# for each scale factor
 		for my $i (0 .. 1) {
 			
-			# if gamut scale factor != 0, and white point defined
-			if ($_[$i] != 0 && defined($self->[1][$i][1])) {
+			# if gamut scale factor not 0, and white point defined
+			if ($_[$i] && defined($self->[1][$i][1])) {
 				
 				# for XYZ
 				for my $j (0 .. 2) {
@@ -1561,7 +1561,7 @@ sub _new_pcs {
 			(3 == grep {! ref()} @{$_[$i][1]}) or croak("$io[$i] white point array has wrong structure");
 			
 			# copy white point (converting XYZNumber to xyz)
-			$self->[1][$i][1] = [$_[$i][1][0]/ICC::Shared::d50->[0], $_[$i][1][1]/ICC::Shared::d50->[1], $_[$i][1][2]/ICC::Shared::d50->[2]];
+			$self->[1][$i][1] = ICC::Shared::XYZ2xyz($_[$i][1], ICC::Shared::d50);
 			
 		} else {
 			
@@ -1580,7 +1580,7 @@ sub _new_pcs {
 			(3 == grep {! ref()} @{$_[$i][2]}) or croak("$io[$i] black point array has wrong structure");
 			
 			# copy black point (converting XYZNumber to xyz)
-			$self->[1][$i][2] = [$_[$i][2][0]/ICC::Shared::d50->[0], $_[$i][2][1]/ICC::Shared::d50->[1], $_[$i][2][2]/ICC::Shared::d50->[2]];
+			$self->[1][$i][2] = ICC::Shared::XYZ2xyz($_[$i][2], ICC::Shared::d50);
 			
 		} else {
 			
